use std::env;
use std::fmt;
use structopt::StructOpt;
use u_lib::{
    api::ClientHandler, datatypes::DataResult, messaging::AsMsg, models::JobMeta, utils::init_env,
    UResult,
};
use uuid::Uuid;

#[derive(StructOpt, Debug)]
struct Args {
    #[structopt(subcommand)]
    cmd: Cmd,
    #[structopt(long)]
    json: bool,
}

#[derive(StructOpt, Debug)]
enum Cmd {
    Agents(LD),
    Jobs(JobALD),
    Jobmap(JobMapALD),
}

#[derive(StructOpt, Debug)]
enum JobALD {
    Add {
        #[structopt(long, parse(try_from_str = parse_uuid))]
        agent: Option<Uuid>,

        #[structopt(long)]
        alias: String,

        #[structopt(subcommand)]
        cmd: JobCmd,
    },
    #[structopt(flatten)]
    LD(LD),
}

#[derive(StructOpt, Debug)]
enum JobCmd {
    #[structopt(external_subcommand)]
    Cmd(Vec<String>),
}

#[derive(StructOpt, Debug)]
enum JobMapALD {
    Add {
        #[structopt(parse(try_from_str = parse_uuid))]
        agent_uid: Uuid,

        job_idents: Vec<String>,
    },
    List {
        #[structopt(parse(try_from_str = parse_uuid))]
        uid: Option<Uuid>,
    },
    Delete {
        #[structopt(parse(try_from_str = parse_uuid))]
        uid: Uuid,
    },
}

#[derive(StructOpt, Debug)]
enum LD {
    List {
        #[structopt(parse(try_from_str = parse_uuid))]
        uid: Option<Uuid>,
    },
    Delete {
        #[structopt(parse(try_from_str = parse_uuid))]
        uid: Uuid,
    },
}

fn parse_uuid(src: &str) -> Result<Uuid, String> {
    Uuid::parse_str(src).map_err(|e| e.to_string())
}

async fn process_cmd(args: Args) {
    struct Printer {
        json: bool,
    }

    impl Printer {
        pub fn print<Msg: AsMsg + fmt::Display>(&self, data: UResult<Msg>) {
            if self.json {
                let data = match data {
                    Ok(r) => DataResult::Ok(r),
                    Err(e) => DataResult::Err(e),
                };
                println!("{}", serde_json::to_string_pretty(&data).unwrap());
            } else {
                match data {
                    Ok(r) => println!("{}", r),
                    Err(e) => eprintln!("Error: {}", e),
                }
            }
        }
    }

    let token = env::var("ADMIN_AUTH_TOKEN").expect("Authentication token is not set");
    let cli_handler = ClientHandler::new(None).password(token);
    let printer = Printer { json: args.json };
    match args.cmd {
        Cmd::Agents(action) => match action {
            LD::List { uid } => printer.print(cli_handler.get_agents(uid).await),
            LD::Delete { uid } => printer.print(cli_handler.del(Some(uid)).await),
        },
        Cmd::Jobs(action) => match action {
            JobALD::Add {
                cmd: JobCmd::Cmd(cmd),
                alias,
                agent: _agent,
            } => {
                let job = JobMeta::builder()
                    .with_shell(cmd.join(" "))
                    .with_alias(alias)
                    .build()
                    .unwrap();
                printer.print(cli_handler.upload_jobs(&[job]).await);
            }
            JobALD::LD(LD::List { uid }) => printer.print(cli_handler.get_jobs(uid).await),
            JobALD::LD(LD::Delete { uid }) => printer.print(cli_handler.del(Some(uid)).await),
        },
        Cmd::Jobmap(action) => match action {
            JobMapALD::Add {
                agent_uid,
                job_idents,
            } => printer.print(cli_handler.set_jobs(Some(agent_uid), &job_idents).await),
            JobMapALD::List { uid } => printer.print(cli_handler.get_agent_jobs(uid).await),
            JobMapALD::Delete { uid } => printer.print(cli_handler.del(Some(uid)).await),
        },
    }
}

#[tokio::main]
async fn main() {
    init_env();
    let args: Args = Args::from_args();
    process_cmd(args).await;
}
